#ifndef _GOST_89_H
#define _GOST_89_H

#include <linux/kernel.h>
#include <linux/export.h>

/* Typedef for unsigned 8-bit integer */
typedef unsigned char byte;

/* Typedef for unsigned 32-bit integer */
#if __LONG_MAX__ > 2147483647L
typedef unsigned int u4;
#else
typedef unsigned long u4;
#endif

#if __LONG_MAX__ > 2147483647L
typedef unsigned int word32;
#else
typedef unsigned long word32;
#endif

/* Internal representation of GOST substitution blocks */
typedef struct {
	byte k8[16];
	byte k7[16];
	byte k6[16];
	byte k5[16];
	byte k4[16];
	byte k3[16];
	byte k2[16];
	byte k1[16];
} gost_subst_block;

/* Cipher context includes key and preprocessed  substitution block */
typedef struct {
	u4 k[8];
	/* Constant s-boxes -- set up in gost_init(). */
	u4 k87[256], k65[256], k43[256], k21[256];
} gost_ctx;

/* CTR mode GOST PRNG */
typedef struct {
	unsigned long ctr;
	gost_ctx ctx;
} gost_rnd_ctx;

/* Note: encrypt and decrypt expect full blocks--padding blocks is
 *	caller's responsibility. All bulk encryption is done in
 *	ECB mode by these calls. Other modes may be added easily
 *	enough.
 */

/* Encrypt several full blocks in ECB mode */
void gost_enc(gost_ctx *, const byte *, byte *, int);

/* Decrypt several full blocks in ECB mode */
void gost_dec(gost_ctx *, const byte *, byte *, int);

/* Encrypt one block */
//static void gostcrypt(gost_ctx *, const byte *, byte *);

/* Decrypt one block */
//static void gostdecrypt(gost_ctx *, const byte *, byte *);

/* Set key into context */
void gost_key(gost_ctx *, const byte *);

/* Get key from context */
void gost_get_key(gost_ctx *, byte *);

/* Set S-blocks into context */
void gost_init(gost_ctx *, const gost_subst_block *);

/* Clean up context */
void gost_destroy(gost_ctx *);

/* Intermediate function used for calculate hash */
void gost_enc_with_key(gost_ctx *, byte *, byte *, byte *);

/* Parameter sets specified in RFC 4357 */
extern gost_subst_block GostR3411_94_TestParamSet;
extern const byte CryptoProKeyMeshingKey[];

void gost_rnd_init(gost_rnd_ctx *, unsigned char *);
u64 gost_rnd(gost_rnd_ctx *);

#endif /* _GOST_89_H */
